import pygame
from pygame.locals import *
from events import *

def debug( msg ):
    print "Debug Message: " + str(msg)

class EventManager:
    
    #This object is responsible for coordinating most communication
    #between the Model, View, and Controller.
    def __init__(self):
        from weakref import WeakKeyDictionary
        self.listeners = WeakKeyDictionary()
        self.eventQueue= []
 
    #----------------------------------------------------------------------
    def register_listener(self, listener):
        self.listeners[listener] = 1
 
    #----------------------------------------------------------------------
    def unregister_listener(self, listener):
        if listener in self.listeners:
            del self.listeners[listener]
 
    #----------------------------------------------------------------------
    def post(self, event):
#        if not isinstance(event, TickEvent) and not isinstance(event, MouseMoveEvent):
#            debug(event.name)
#NOTE: copying the list like this before iterating over it, EVERY tick, is highly inefficient,
#but currently has to be done because of how new listeners are added to the queue while it is running
#(eg when popping cards from a deck). Should be changed. See: http://dr0id.homepage.bluewin.ch/pygame_tutorial08.html
#and search for "Watch the iteration"
        for listener in list(self.listeners):
            #NOTE: If the weakref has died, it will be 
            #automatically removed, so we don't have 
            #to worry about it.
            listener.notify(event)
 
#------------------------------------------------------------------------------
class PyGameEventController:
    """..."""
    def __init__(self, ev_manager):
        self.ev_manager = ev_manager
        self.ev_manager.register_listener(self) 
        
    #----------------------------------------------------------------------
    def notify(self, incoming_event):

        if isinstance(incoming_event, TickEvent):
            
            #Handle Pygame Events
            for event in pygame.event.get():
                ev = None
                if event.type == QUIT:
                    ev = QuitEvent()
                elif event.type == pygame.MOUSEBUTTONDOWN:
                    if event.button == 1:    #Button 1
                        pos = pygame.mouse.get_pos()
                        ev = MouseButtonLeftEvent(pos)
                elif event.type == pygame.MOUSEMOTION:
                        pos = pygame.mouse.get_pos()
                        ev = MouseMoveEvent(pos)
                        
                #Post event to event manager
                if ev:
                    self.ev_manager.post(ev)                          
  
 
#------------------------------------------------------------------------------            
class CPUSpinnerController:
    
    def __init__(self, ev_manager):
        self.ev_manager = ev_manager
        self.ev_manager.register_listener(self)
 
        self.keep_going = True
 
    #----------------------------------------------------------------------
    def run(self):
        if not self.keep_going:
            raise Exception('dead spinner')        
        while self.keep_going: 
            event = TickEvent()
            self.ev_manager.post(event)
        
        pygame.quit()
            
 
    #----------------------------------------------------------------------
    def notify(self, event):
        if isinstance(event, QuitEvent):
            #this will stop the while loop from running
            self.keep_going = False            
            
#class MouseController:
#    #MouseController takes Pygame events generated by the
#    #mouse and uses them to control the model, by sending Requests
#    #or to control the Pygame display directly, as with the QuitEvent
#    def __init__(self, ev_manager):
#        self.ev_manager = ev_manager
#        self.ev_manager.register_listener(self)
# 
#    #----------------------------------------------------------------------
#    def notify(self, event):
#        if isinstance(event, TickEvent):
#            #Handle Input Events
#            for event in pygame.event.get():
#                ev = None
#                if event.type == pygame.MOUSEBUTTONDOWN:
#                    if event.button == 1:    #Button 1
#                        pos = pygame.mouse.get_pos()
#                        ev = MouseButtonLeftEvent(pos)
#                elif event.type == pygame.MOUSEMOTION:
#                        pos = pygame.mouse.get_pos()
#                        ev = MouseMoveEvent(pos)
#                
#                #Post event to event manager
#                if ev:
#                    self.ev_manager.post(ev)  
#        else:
#            debug( event.name )
            
#class RenderController:
#    def __init__(self, screen, ev_manager):
#        self.ev_manager = ev_manager
#        self.ev_manager.register_listener(self)
# 
#        self.screen = screen
#        
#    #----------------------------------------------------------------------
#    def notify(self, event):
#        if isinstance(event, TickEvent):
#            #Draw Everything
#            event = RenderEvent()
#            self.ev_manager.post(event)
#            pygame.display.update()