import pygame
from pygame.locals import *
from math import sqrt, ceil, floor
from world import World
from matrix import Matrix
import string
from textrect import render_textrect
import random

#CONSTANTS TO CHANGE
SCREEN_SIZE = (1024, 576)
CARD_SIZE = (70, 70)
CARD_MARGIN = (10, 10)
SCREEN_MARGIN = (10, 10)

BLACK = (0, 0, 0)
WHITE = (255, 255, 255)
RED = (255, 0, 0)

GRAY1 = (182, 182, 182)
GRAY2 = (155, 155, 155)
GRAY3 = (126, 126, 126)
GRAY4 = (92, 92, 92)
GRAY5 = (72, 72, 72)

class Interface(object):
    
    def __init__(self):
        pass

class Board(object):
    
    def __init__(self, continent):
        
        #Determine how many columns of cards to make, based on number of countries
        #in the continent
        #self.matrix_size[0] = min(7, int(ceil(sqrt(continent.size))))
        #self.matrix_size[1] = min(6, int(ceil(sqrt(continent.size)))
        self.matrix_size = [0, 0]
        self.matrix_size[0] = 5    #Number of rows
        self.matrix_size[1] = 5     #Number of columns
        
        #Determine the pixel width of the card area, based on card size, and number of cards
        self.board_width = self.matrix_size[1] * (CARD_SIZE[0] + CARD_MARGIN[0])
        
        #Create a matrix to hold the cards, and place cards in initial positions
        self.matrix = Matrix(self.matrix_size[0], self.matrix_size[1])

        self.countries = continent.countries
        random.shuffle(self.countries)
        self.deal_cards()
                
        #Create initial connections
        self.refresh_connections()          
                       
        #Initialize other properties       
        self.background = pygame.surface.Surface(SCREEN_SIZE).convert()
        self.background.fill((255, 255, 255))   
        self.map_image = pygame.image.load(continent.map).convert()
        self.map_image_pos = (SCREEN_SIZE[0] - self.map_image.get_width() - SCREEN_MARGIN[0], 2*SCREEN_MARGIN[1] + CARD_SIZE[1])
        self.selected_card = None
        
        print self.matrix
        
    def deal_cards(self):
        count = 0;
        card_pos = [0, 0]
        for i in xrange(self.matrix_size[0]):
            for j in xrange(self.matrix_size[1]):
                if count < len(self.countries):
                    current_country = self.countries[count]
                    card_pos[0] = j * (CARD_SIZE[0] + CARD_MARGIN[0]) + SCREEN_MARGIN[0]
                    card_pos[1] = i * (CARD_SIZE[1] + CARD_MARGIN[1]) + SCREEN_MARGIN[1]              
                    self.matrix.setitem(i, j, Card('country', current_country, card_pos, [i, j]))
                    count += 1;
                else:
                    break
        print self.get_card([0,0])
            
    #Return the card at the given matrix index (r, c)            
    def get_card(self, index):
        return self.matrix.getitem(index[0], index[1])
    
    def try_swap(self, card1, card2):
        #Change card information for testing
        card1_index_old = card1.index
        card2_index_old = card2.index
        card1.index = card2_index_old
        card2.index = card1_index_old    
        
        #Test for new connections after the swap
        any_connections = card1.test_any_connections(self) or card2.test_any_connections(self)
        
        print any_connections
        
        #Change card info back
        card1.index = card1_index_old
        card2.index = card2_index_old  
        
        #If there are connections after the swap, then swap, else don't
        if any_connections:       
            self.swap(card1, card2) 
        else: 
            return False        
        
    def swap(self, card1, card2):
        #change card information
        card1_index_old = card1.index
        card2_index_old = card2.index
        card1.index = card2_index_old
        card2.index = card1_index_old    
        
        card1_pos_old_0 = card1.pos[0]
        card1_pos_old_1 = card1.pos[1]
        card2_pos_old_0 = card2.pos[0]
        card2_pos_old_1 = card2.pos[1]
        card1.set_pos([card2_pos_old_0, card2_pos_old_1])
        card2.set_pos([card1_pos_old_0, card1_pos_old_1])
          
        card1.selected = False
        card2.selected = False
        
        #change board information
        self.selected_card = None 
        self.matrix.setitem(card1_index_old[0], card1_index_old[1], card2)
        self.matrix.setitem(card2_index_old[0], card2_index_old[1], card1)
        self.refresh_connections()  
    
    #refresh all connections on the board - typically done when a card is moved
    def refresh_connections(self):
        self.connections = []
        for card,r,c in self.matrix:
            if card[r][c] != 0:
                current_card = card[r][c]
                try:
                    card_right = card[r][c+1]
                    if current_card.test_connection(card_right):
                        new_connection = Connection(current_card, card_right)
                        self.connections.append(new_connection)
                except:
                    pass
                try:
                    card_down = card[r+1][c]
                    if current_card.test_connection(card_down):
                        new_connection = Connection(current_card, card_down)
                        self.connections.append(new_connection)
                except:
                    pass          
    
    #Render all board components
    def render(self, surface):
        surface.blit(self.background, (0, 0))
        surface.blit(self.map_image, (self.map_image_pos))
        #Render connections
        for connection in self.connections:
            connection.render(surface)            
        #Render cards
        for card,r,c in self.matrix:
            if card[r][c] != 0:
                card[r][c].render(surface)           
    
class Card(object):
    
    def __init__(self, type, value, pos, index):
        self.pos = [0, 0]
        self.text_pos = [0, 0] 
        self.image_pos = [0, 0]       
        self.index = [0, 0]    #board.matrix index
        self.index[0] = index[0]
        self.index[1] = index[1]
        self.type = type
        self.value = value    
        self.country_size = value.size
        self.color = value.color
        self.borders = value.borders
        self.name = value.name
        self.machine_name = string.lower(string.replace(self.name, ' ', '-')) 
        self.font = pygame.font.Font("assets/fonts/verdana.TTF", 8)
        self.text = self.name
        self.border_width = 1;        
        self.text_margin = (self.border_width + 1, self.border_width + 2);
        self.text_size = self.font.size(self.name)
        self.text_color = BLACK;
        self.text_bg_color = WHITE;
        self.rendered_text = self.render_text(self.text)
        self.selected = False
        self.init_image() 
        self.set_pos(pos)     
               
    def init_image(self):
        try:
            self.image_size = [0, 0]
            self.image = pygame.image.load("assets/images/countries/europe/" + self.machine_name + ".png").convert_alpha()  
            self.image_pixel_array = pygame.PixelArray(self.image)
            #Make the country the correct color
            self.image_pixel_array.replace(GRAY1, self.color)
            self.image_pixel_array.replace(GRAY2, self.color)
            self.image_pixel_array.replace(GRAY3, self.color)
            self.image_pixel_array.replace(GRAY4, self.color)
            self.image_pixel_array.replace(GRAY5, self.color)
            height_ratio = float(self.image.get_height()) / float(self.image.get_width())
            #print height_ratio
            max_image_height = int(self.country_size * (CARD_SIZE[1] - self.text_size[1]*self.text_lines - 2*self.text_margin[1]))
            max_image_width = int(self.country_size * (CARD_SIZE[0] - 2*self.text_margin[0]))
            if height_ratio >= float(max_image_height) / float(max_image_width):
                self.image_size[1] = max_image_height
                self.image_size[0] = int(self.image_size[1] / height_ratio) 
            else:
                self.image_size[0] = max_image_width
                self.image_size[1] = int(self.image_size[0] * height_ratio)
                
            #self.image_size[0] = self.image_size[1] = int(self.country_size * (CARD_SIZE[1] - self.text_size[1]*self.text_lines - 2*self.text_margin[1] - self.border_width))
            self.image = pygame.transform.smoothscale(self.image, self.image_size)
        except:
            self.image = None      
       
    def render_text(self, text):
        if self.text_size[0] < (CARD_SIZE[0] - 2*self.text_margin[0]):
            self.text_lines = 1;
        else:
            self.text_lines = 2;
        self.text_rect = pygame.Rect(0, 0, CARD_SIZE[0] - 2*self.text_margin[0], 1+self.text_lines*self.text_size[1])
        return render_textrect(self.text, self.font, self.text_rect, self.text_color, self.text_bg_color, 1)       
        
    def set_pos(self, pos):
        self.pos[0] = pos[0]
        self.pos[1] = pos[1]
        #self.text_pos = (0,0)
        self.text_pos[0] = self.pos[0] + self.text_margin[0]
        self.text_pos[1] =  self.pos[1] + CARD_SIZE[1] - self.text_margin[1] - self.text_size[1]*self.text_lines
        try:
            self.image_pos[0] = self.get_center()[0] - self.image_size[0]/2
            self.image_pos[1] = self.get_center()[1] - self.text_size[1] - self.image_size[1]/2 + 2*self.text_margin[1]
        except:
            pass
        
    def get_pos(self):
        return self.pos
    
    def get_center(self):
        return (self.pos[0] + CARD_SIZE[0]/2, self.pos[1] + CARD_SIZE[1]/2)
        
    #Test to see if a given position (x,y) is covered by this card
    def test_pos_on_card(self, pos):
        if pos[0] > self.pos[0] and pos[0] < self.pos[0] + CARD_SIZE[0]:
            if pos[1] > self.pos[1] and pos[1] < self.pos[1] + CARD_SIZE[1]:
                return True
        return False
    
    #Test whether a particular card is next to this one
    def test_next_to(self, card):
        if abs(self.index[0] - card.index[0]) == 1 and self.index[1] == card.index[1]:
            return True
        elif abs(self.index[1] - card.index[1]) == 1 and self.index[0] == card.index[0]:
            return True
        else:
            return False
        
    #Test whether there is a connection between a particular card and this one    
    def test_connection(self, card):
        for country_name in self.borders:
            if country_name == card.name:
                return True
        return False
    
    #Test whether this card has any connections, in it's current location
    #Returns True or False
    def test_any_connections(self, board):
        r = self.index[0]
        c = self.index[1]
        try: 
            card_right = board.get_card([r, c+1])
            print card_right
            if self.test_connection(card_right):
                return True
        except:
            pass
        try: 
            card_down = board.get_card([r+1, c])
            if self.test_connection(card_down):
                return True
        except:
            pass
        try: 
            card_left = board.get_card([r, c-1])
            if self.test_connection(card_left):
                return True
        except:
            pass  
        try: 
            card_up = board.get_card([r-1, c])
            if self.test_connection(card_up):
                return True
        except:
            pass    
        return False             
            
    def __repr__(self):
#        output = ""
#        output += "Name: " + str(self.name) + "\n"
#        output += "Borders: " + str(self.borders) + "\n"
#        output += "Index: " + str(self.index) + "\n"        
#        output += "Pos: " + str(self.pos) + "\n"
#        output += "selected: " + str(self.selected) + "\n"        
#        return output 
        return str(self.name)
    
    def render(self, surface):  
        if self.selected:
            if self.image:
                pygame.draw.rect(surface, (0,0,0), Rect(self.pos, (CARD_SIZE[0], CARD_SIZE[1])))
                surface.blit(self.image, self.image_pos, None, 0)
        else:
            if self.image:
                #pygame.draw.rect(surface, self.color, Rect(self.pos, (CARD_SIZE[0], CARD_SIZE[1])))
                
                pygame.draw.rect(surface, WHITE, Rect(self.pos, (CARD_SIZE[0], CARD_SIZE[1])))
                surface.blit(self.image, self.image_pos, None, 0)
                #pygame.draw.rect(surface, RED, Rect(self.image_pos, (self.image_size[0], self.image_size[1])), 1)
                pygame.draw.rect(surface, BLACK, Rect(self.pos, (CARD_SIZE[0], CARD_SIZE[1])), 1)
            else:
                pygame.draw.rect(surface, self.color, Rect(self.pos, (CARD_SIZE[0], CARD_SIZE[1])))
        surface.blit(self.rendered_text, self.text_pos)
    
class Connection(object):    
    
    def __init__(self, card1, card2):
        self.card1 = card1
        self.card2 = card2
        self.point_a = card1.get_center()
        self.point_b = card2.get_center()
        
    def __repr__(self):
        return self.card1.name + "--" + self.card2.name
    
    def render(self, surface):
        pygame.draw.aaline(surface, (0, 0, 0), self.point_a, self.point_b)
    
#Run game 
def main():
    pygame.init()
    screen = pygame.display.set_mode(SCREEN_SIZE)
    europe = World().europe
    board = Board(europe)
        
    #Main game loop    
    while True:
        for event in pygame.event.get():
            if event.type == QUIT:
                return       
            #Handle mouse clicks
            elif event.type == pygame.MOUSEBUTTONDOWN:
                print "*******************/n"              
                for card,r,c in board.matrix:
                    if card[r][c] != 0 and card[r][c].test_pos_on_card(event.pos):
                        if board.selected_card and card[r][c].selected == False and card[r][c].test_next_to(board.selected_card):
                            board.try_swap(card[r][c], board.selected_card)
                        elif card[r][c].selected:
                            card[r][c].selected = False
                            board.selected_card = None
                        elif not board.selected_card:
                            card[r][c].selected = True         
                            board.selected_card = card[r][c] 
#                    print card[r][c]       
#                print board.connections      
        
        board.render(screen)
        
        pygame.display.update()

# this runs the main function if this script is called to run.
#  If it is imported as a module, we don't run the main function.
if __name__ == "__main__":
    main()