import pygame
from pygame.locals import *
from math import sqrt, ceil, floor
from world import World
from matrix import Matrix
import string
from textrect import render_textrect
import random
from copy import copy, deepcopy

##############################
#CONSTANTS
##############################

#Screen
SCREEN_SIZE = (1024, 576)
SCREEN_MARGIN = (10, 10)

#Colors
BLACK = (0, 0, 0)
WHITE = (255, 255, 255)
RED = (255, 0, 0)

GRAY1 = (182, 182, 182)
GRAY2 = (155, 155, 155)
GRAY3 = (126, 126, 126)
GRAY4 = (92, 92, 92)
GRAY5 = (72, 72, 72)

GRAYS = [GRAY1, GRAY2, GRAY3, GRAY4, GRAY5]

#Directions
LEFT = -1
RIGHT = 1
UP = 2
DOWN = -2 

#Board components
BOARD_SIZE = (7, 6)
AUCTION_BLOCK_SIZE = 4

#Cards
CARD_SIZE = (70, 70)
CARD_MARGIN = (10, 10)
CARD_PADDING = (3, 3)

#Card types
BLANK = 0
COUNTRY = 1
TRANSPORT = 2

#Transport types
PLANE = 0
TRAIN = 1
CAR = 2
SHIP = 3

class Interface(object):
    
    def __init__(self, continent):
        self.board = Board(BOARD_SIZE)
        self.deck = Deck(continent)
        self.auction_block = AuctionBlock()
        self.map = Map(continent)
        self.background = pygame.surface.Surface(SCREEN_SIZE).convert()
        self.background.fill((255, 255, 255)) 
        
        #Set pos of interface elements
        board_pos = (SCREEN_MARGIN[0], SCREEN_MARGIN[1])
        self.board.set_pos(board_pos)
        map_pos = (SCREEN_SIZE[0] - self.map.size[0] - SCREEN_MARGIN[0], 2*SCREEN_MARGIN[1] + CARD_SIZE[1]);
        self.map.set_pos(map_pos)
        auction_block_pos = (map_pos[0] + CARD_MARGIN[0], map_pos[1])
        self.auction_block.set_pos(auction_block_pos)        
        
        #Deal initial cards into board from deck
        self.deck.shuffle()
        self.deck, self.board = self.deck.deal_connection_matrix(self.board, (0,0), 10)
 
    def process(self):    

#        #Click anywhere else
#        for count in xrange(100):
#            print 'Count: ' + str(count)
#            self.deck.merge_board_stack(self.board)
#            print 'Deck after merging board stack: ' + str(len(self.deck.stack))
##            if len(self.deck.stack) != 52:
##                exit()
#            self.deck.shuffle()
#            self.deck, self.board = self.deck.deal_connection_matrix(self.board, (0,0), 20)
#            deck_after_dealing_connection = str(len(self.deck.stack))
#            board_after_dealing_connection = self.board.count_cards()
#            board_matrix_after_dealing_connection = self.board.matrix
#            print 'Deck after dealing connection matrix: ' + str(deck_after_dealing_connection)
#            print 'Board after dealing connection matrix: ' + str(board_after_dealing_connection)
##            self.board = self.board.displace_cards(1, [(0, 0)])     
#            board_after_displace = self.board.count_cards()
#            board_matrix_after_displace = self.board.matrix
#            print 'Board displacing cards: ' + str(board_after_displace)
##            if board_after_displace != board_after_dealing_connection:
##                print 'Lost ' + str(board_after_dealing_connection - board_after_displace) + ' cards after displacing.'
##                print board_matrix_after_dealing_connection
##                print board_matrix_after_displace
##                exit() 
#        print 'Finished'
#        exit()                        

        for event in pygame.event.get():
            #Handle quit
            if event.type == QUIT:
                exit()       
            #Handle mouse clicks
            elif event.type == pygame.MOUSEBUTTONDOWN:
                #Click on auction_block card
                for card in self.auction_block.array:
                    if card.test_pos_on_card(event.pos):
                        self.deck.deal_to_blank_squares(self.board)
                        return
 
                #Click on board card
                for card, r, c in self.board.matrix:
                    if card[r][c].test_pos_on_card(event.pos):
                        if self.board.selected_card and card[r][c].selected == False and card[r][c].test_next_to(self.board.selected_card):
                            self.board.try_swap(card[r][c], self.board.selected_card)
                        elif card[r][c].selected:
                            card[r][c].selected = False
                            self.board.selected_card = None
                        elif not self.board.selected_card:
                            card[r][c].selected = True         
                            self.board.selected_card = card[r][c]   
                        return            

                #Click anywhere else
                self.deck.merge_board_stack(self.board)
                deck_after_merge = len(self.deck.stack)
                if deck_after_merge != 52:
                    print '*ALERT* Deck after merge: ' + str(deck_after_merge)             
                self.deck.shuffle()
                self.deck, self.board = self.deck.deal_connection_matrix(self.board, (0,0), 10)
#                self.board = self.board.displace_cards(1, [(0, 0)])  
                cards_on_board = self.board.count_cards()
                if cards_on_board != 11:
                    print '*ALERT* cards on board: ' + str(cards_on_board)
        
    def render(self, surface):
        surface.blit(self.background, (0, 0))
        self.map.render(surface)        
        self.board.render(surface)
        self.auction_block.render(surface)

class Map(object):
    def __init__(self, continent):
        self.map_image = pygame.image.load(continent.map).convert()
        self.size = self.map_image.get_size()
        self.pos = (0, 0)
        
    def set_pos(self, pos):
        self.pos = pos    
        
    def render(self, surface):
        surface.blit(self.map_image, (self.pos))

class Deck(object):
    
    def __init__(self, continent = None):
        self.stack = []
        self.discard_stack = []        
        self.continent = continent  
        if continent != None:  
            self.rebuild(continent)  
            
    def clone(self):
        new_deck = Deck()
        for card in self.stack:
            new_card = card.clone()
            new_deck.stack.append(new_card)
        for card in self.discard_stack:
            new_card = card.clone()
            new_deck.discard_stack.append(new_card)
        return new_deck
        
        new_deck.stack = self.stack
        new_deck.discard_stack = self.discard_stack
        return new_deck      
    
    def rebuild(self, continent):
        self.stack = []
        self.discard_stack = []
                
        #Add countries  
        for country in continent.countries:
            for _ in xrange(country.number):
                self.stack.append(CountryCard(country))
        
        #Add ships
        for ocean in continent.oceans:
            for _ in xrange(ocean.number):
                self.stack.append(ShipCard(ocean))
                
        #Add planes
        for color in continent.colors:
            for _ in xrange(1):
                self.stack.append(PlaneCard(color))
                
        #Add cars
#        for _ in xrange(5):
#            self.stack.append(CarCard())        
        
    def shuffle(self):
        random.shuffle(self.stack)
            
    def deal(self, board):
        for _, r, c in board.matrix:
            old_card = board.get_card((r, c))
            try:
                new_card = self.stack.pop(0)
                new_card.set_pos(old_card.pos)
                board.place_card(new_card, (r,c))
            except IndexError:
                self.merge_discard_stack()
                self.shuffle()
                new_card = self.stack.pop(0)
                new_card.set_pos(old_card.pos)  
                board.place_card(new_card, (r,c))             
            if old_card.type != BLANK:    
                self.discard_stack.append(old_card)          
    
    #Returns the given board with a connection sequence of the specified number of connections   
    def deal_connection_matrix(self, board, starting_index, number_of_connections):
        number_of_tries = 0
        try_again = True
        while try_again == True:
            number_of_tries += 1
            print 'Connection Matrix Number of tries: ' + str(number_of_tries)          
            if number_of_tries >= 1000:
                raise Exception('Was unable to create connection matrix after ' + str(number_of_tries) + ' tries.' )
                exit()            

            board_clone = board.clone()
            deck_clone = self.clone()    
            deck_clone.shuffle()       
            current_row = starting_index[0]
            current_col = starting_index[1]
            protected_indexes = []                
            
            #Set starting card
            for card in deck_clone.stack:
                if card.type == COUNTRY:
                    deck_clone.place_card_on_board(card, board_clone, (current_row, current_col))
                    protected_indexes.append((current_row, current_col))
                    last_card = card
                    break       
                
            #Create connections
            test_counter = 0
            for i in xrange(number_of_connections):
                test_counter += 1
                
                #Determine which direction to go for connection
                possible_new_indexes = []    
                adjacent_indexes = last_card.get_adjacent_indexes(board_clone)
                for adjacent_index in adjacent_indexes:
                    if adjacent_index not in protected_indexes:
                        possible_new_indexes.append(adjacent_index)
                if len(possible_new_indexes) == 0:      
                    print 'possible new index trouble'         
                    try_again = True
                    break               
                random.shuffle(possible_new_indexes)
                current_row, current_col = possible_new_indexes.pop()     
                
                #Find and place card with connection to last card 
                for card in deck_clone.stack:
                    found_connection = False
                    card.set_index((current_row, current_col))
                    if i == (number_of_connections - 1) and card.type == TRANSPORT:                   
                        continue
                    if card.type == TRANSPORT and card.transport_type == CAR:   #Special case for cars
                        if last_card.type == COUNTRY:
                            found_connection = True
                            break           
                    if card.test_connection(last_card, board_clone):
                        found_connection = True
                        break
                    
                if found_connection == True:
                    try_again = False
                    deck_clone.place_card_on_board(card, board_clone, (current_row, current_col))              
                    protected_indexes.append((current_row, current_col))  
                    last_card = card     
                else: 
                    try_again = True
                    break

        return (deck_clone, board_clone)
    
    def deal_to_blank_squares(self, board):
        for card, r, c in board.matrix:
            if card[r][c].type == BLANK:
                new_card = self.stack.pop()
                board.place_card(new_card, (r, c))
        
    def discard(self, card):
        self.stack.remove(card)
        self.discard_stack.append(card)   

    def place_card_on_board(self, card, board, index):
        board.place_card(card, index)
        self.stack.remove(card)
        
    def contains(self, card):
        try:
            return self.stack.index(card)
        except ValueError:
            return False
        
    def merge_discard_stack(self):
        for card in self.discard_stack:
            self.stack.append(card)
        self.discard_stack = []
        
    def merge_board_stack(self, board):
        for card, r, c in board.matrix:
            if card[r][c].type != BLANK:
                self.stack.append(card[r][c])
                board.remove((r, c))
        
class Board(object):
    
    def __init__(self, size):
        self.pos = (0, 0)
        self.matrix_size = size
        self.connections = []
        self.selected_card = None   
        
        #Create a matrix to hold the cards, and deal blank cards to start
        self.matrix = Matrix(self.matrix_size[0], self.matrix_size[1])
        self.clear()
        
    def clone(self):
        new_board = Board((self.matrix_size[0], self.matrix_size[1]))
        new_board.set_pos(self.pos)
        for card, r, c in self.matrix:
            new_card = card[r][c].clone()
            new_board.place_card(new_card, (r, c))
        return new_board
               
     
    def count_cards(self):
        count = 0
        for card, r, c in self.matrix:
            if card[r][c].type != BLANK:
                count += 1
        return count
     
    def clear(self):
        for _, r, c in self.matrix:
            blank_card = BlankCard()
            blank_card.set_index([r, c]) 
            self.matrix.setitem(r, c, blank_card)     
        self.set_pos(self.pos)
        self.connections = []
        
    #Replace card at given index with blank card
    def remove(self, index):
        blank_card = BlankCard()
        self.matrix.setitem(index[0], index[1], blank_card) 
        blank_card.set_index((index[0], index[1]))     
        self.set_pos(self.pos)
        self.connections = self.refresh_connections()
        
    def set_pos(self, pos):
        self.pos = pos    
        for card,r,c in self.matrix:
            x = c * (CARD_SIZE[0] + CARD_MARGIN[0]) + self.pos[0]
            y = r * (CARD_SIZE[1] + CARD_MARGIN[1]) + self.pos[1] 
            card[r][c].set_pos((x,y))  
            
    def place_card(self, card, index):
        self.matrix.setitem(index[0], index[1], card) 
        card.set_index((index[0], index[1]))      
        self.set_pos(self.pos) 
        self.refresh_connections()
            
    #Return the card at the given matrix index (r, c)            
    def get_card(self, index):
        if index[0] < 0 or index[1] < 0:
            raise Exception('Cannot get card from a negative index')
        return self.matrix.getitem(index[0], index[1])    
    
    def test_index_on_board(self, index):
        if index[0] < 0 or index[0] > self.matrix_size[0] - 1:
            return False
        elif index[1] < 0 or index[1] > self.matrix_size[1] - 1:
            return False
        else:
            return True
   
    def try_swap(self, card1, card2):
        #Change card information for testing
        card1_index_old = card1.index
        card2_index_old = card2.index
        card1.index = card2_index_old
        card2.index = card1_index_old    
        
        #Test for new connections after the swap
        any_connections = card1.test_any_connections(self) or card2.test_any_connections(self)
        
        #Change card info back
        card1.index = card1_index_old
        card2.index = card2_index_old  
        
        #If there are connections after the swap, then swap, else don't
        if any_connections:       
            self.swap(card1, card2) 
        else: 
            return False        
        
    def swap(self, card1, card2):
        #change card information
        card1_index_old = card1.index
        card2_index_old = card2.index
        card1.index = card2_index_old
        card2.index = card1_index_old    
        
        card1_pos_old_0 = card1.pos[0]
        card1_pos_old_1 = card1.pos[1]
        card2_pos_old_0 = card2.pos[0]
        card2_pos_old_1 = card2.pos[1]
        card1.set_pos([card2_pos_old_0, card2_pos_old_1])
        card2.set_pos([card1_pos_old_0, card1_pos_old_1])
          
        card1.selected = False
        card2.selected = False
        
        #change board information
        self.selected_card = None 
        self.matrix.setitem(card1_index_old[0], card1_index_old[1], card2)
        self.matrix.setitem(card2_index_old[0], card2_index_old[1], card1)
        self.refresh_connections()     
        
    #displace_by = number of cards to displace each card by  
    #cards with indexes in protected_indexes are not displaced
    #returns False if unsuccessful at displacing cards; can try again
    def displace_cards(self, displace_by, protected_indexes):
        number_of_tries = 0
        try_again = True
        while try_again == True:   
            
            #Throw exception if too many tries
            number_of_tries += 1
            print 'Number of displacement tries: ' + str(number_of_tries)
            if number_of_tries >= 1000:
                raise Exception('Was unable to create displacement board ' + str(number_of_tries) + ' tries.' )
                exit()        
            
            #Set up variables    
            board_clone = self.clone()
            new_board = Board((board_clone.matrix_size[0], board_clone.matrix_size[1]))
            new_board.set_pos(board_clone.pos)
            new_protected_indexes = []    
            last_direction = None
            
            for index in protected_indexes:
                protected_card = board_clone.get_card(index)
                new_board.place_card(protected_card, index)
                new_protected_indexes.append(index)
                
            for card, r, c in board_clone.matrix:
                if card[r][c].type != BLANK and card[r][c].index not in protected_indexes:      
                    
                    #Determine which direction to go for displacement
                    possible_new_indexes = []    
                    row = card[r][c].index[0]
                    col = card[r][c].index[1]
                    if row != 0 and last_direction != UP:
                        if (row - 1, col) not in new_protected_indexes:
                            possible_new_indexes.append((row - 1, col))
                            last_direction = UP
                    if row != board_clone.matrix.last_row and last_direction != DOWN: 
                        if (row + 1, col) not in new_protected_indexes:
                            possible_new_indexes.append((row + 1, col))
                            last_direction = DOWN
                    if col != 0 and last_direction != LEFT:
                        if (row, col - 1) not in new_protected_indexes:
                            possible_new_indexes.append((row, col - 1))
                            last_direction = LEFT
                    if col != board_clone.matrix.last_col and last_direction != RIGHT: 
                        if (row, col + 1) not in new_protected_indexes:
                            possible_new_indexes.append((row, col + 1))
                            last_direction = RIGHT
                        
#                    print possible_new_indexes   
#                    count = 0 
#                    for index in possible_new_indexes:
#                        count += 1
#                        if index in new_protected_indexes:
#                            print 'yeppers'
#                            possible_new_indexes.remove(index)
#                    print 'Times through: ' + str(count)
#                    print new_protected_indexes        
#                    print possible_new_indexes 
                    
                    if len(possible_new_indexes) == 0:
                        try_again = True
                        break 
                    else:
                        try_again = False             
                        random.shuffle(possible_new_indexes)
                        new_index = possible_new_indexes.pop()
                        new_board.place_card(card[r][c], new_index)
                        new_protected_indexes.append(new_index)   
         
#        print new_protected_indexes                     
        return new_board       
                
    
    #refresh all connections on the board - typically done when a card is moved
    def refresh_connections(self):
        self.connections = []
        for card, r, c in self.matrix:
            current_card = card[r][c]
            try:
                card_right = card[r][c+1]
                if current_card.test_connection(card_right, self):
                    new_connection = Connection(current_card, card_right)
                    self.connections.append(new_connection)
            except:
                pass
            try:
                card_down = card[r+1][c]
                if current_card.test_connection(card_down, self):
                    new_connection = Connection(current_card, card_down)
                    self.connections.append(new_connection)
            except:
                pass       
    
    #Render board components
    def render(self, surface):
        #Render connections
        for connection in self.connections:
            connection.render(surface)            
        #Render cards
        for card,r,c in self.matrix:
            if card[r][c] != 0:
                card[r][c].render(surface)             
    
class AuctionBlock(object):  
    
    def __init__(self):
        self.array = []
        self.pos = (0, 0)
        self.selected_card = None
        for _ in xrange(AUCTION_BLOCK_SIZE):
            self.array.append(BlankCard())

    def set_pos(self, pos):
        self.pos = pos    
        for index, card in enumerate(self.array):
            x = index * (CARD_SIZE[0] + CARD_MARGIN[0]) + self.pos[0]
            y = CARD_MARGIN[1] + self.pos[1]             
            card.set_pos((x, y))
    
    def select_card(self, card):
        for card_i in self.array:
            if card_i == card:
                card_i.selected = True
                self.selected_card = card_i
            else:
                card_i.selected = False
                
    def remove_card(self, card):
        self.array.remove(card)
        self.array.append(BlankCard())
        self.set_pos(self.pos)
        
    def __repr__(self):
        return str(self.array)
        
    def render(self, surface):
        for card in self.array:
            card.render(surface)
    
class Card(object):
    
    def __init__(self, type, text, image, color):  
        self.price = 0
        self.font = pygame.font.Font("assets/fonts/verdana.TTF", 10)
        self.text_color = BLACK
        self.text_bgcolor = WHITE             
        self.pos = [0, 0]       
        self.type = type     
        self.set_text(text)
        self.color = color
        self.set_image(image)   
        self.selected = False  
        
    def clone(self):
        if self.type == BLANK:
            return BlankCard()
        elif self.type == COUNTRY:
            return CountryCard(self.country)
        elif self.type == TRANSPORT:
            if self.transport_type == PLANE:
                return PlaneCard(self.color)
            elif self.transport_type == TRAIN:
                return TrainCard()
            elif self.transport_type == CAR:
                return CarCard()    
            elif self.transport_type == SHIP:
                return ShipCard(self.ocean)                    
        
    def set_text(self, text):
        self.text = text
        self.rendered_text = self.render_text_rec(text)  
        self.text_size = self.rendered_text.get_size()    
        
        self.rendered_price = self.font.render("$" + str(self.price), True, BLACK, WHITE)        

    def set_image(self, image):
        self.image = image
        try:
            image = pygame.image.load(image).convert_alpha()  
            
            #Correct Color
            if self.color:
                image_pixel_array = pygame.PixelArray(image)
                try:
                    image_pixel_array.replace(GRAY1, self.color)
                    image_pixel_array.replace(GRAY2, self.color)
                    image_pixel_array.replace(GRAY3, self.color)
                    image_pixel_array.replace(GRAY4, self.color)
                    image_pixel_array.replace(GRAY5, self.color)
                except pygame.error, e:
                    print e
                    exit()
            
            #Scale image to card size
            height_ratio = float(image.get_height()) / float(image.get_width())
            max_image_height = int(CARD_SIZE[1] - self.text_size[1] - 2*CARD_PADDING[1])
            max_image_width = int(CARD_SIZE[0] - 2*CARD_PADDING[0])
            self.image_size = [0, 0]
            if height_ratio >= float(max_image_height) / float(max_image_width):
                self.image_size[1] = max_image_height
                self.image_size[0] = int(self.image_size[1] / height_ratio) 
            else:
                self.image_size[0] = max_image_width
                self.image_size[1] = int(self.image_size[0] * height_ratio)
                
            #Scale image with scale_factor
            if self.type == COUNTRY:
                scale_factor = self.country_size
            else:
                scale_factor = 0.9                
            self.image_size[0] = int(scale_factor*self.image_size[0])
            self.image_size[1] = int(scale_factor*self.image_size[1])
            
            self.rendered_image = pygame.transform.smoothscale(image, self.image_size)
        except pygame.error, e:
            print e
            exit()
        
    def set_pos(self, pos):
        #Set card pos
        self.pos[0] = pos[0]
        self.pos[1] = pos[1]
        
        #Set text pos
        self.text_pos = [0, 0]
        self.text_pos[0] = self.pos[0] + CARD_PADDING[0]
        self.text_pos[1] = self.pos[1] + CARD_SIZE[1] - CARD_PADDING[1] - self.text_size[1]
        
        #Set price pos
        self.price_pos = [0, 0]
        self.price_pos[0] = self.pos[0] + CARD_PADDING[0]
        self.price_pos[1] = self.pos[1] + CARD_PADDING[1]
        
        #Set img pos
        try:
            self.image_pos = [0, 0]
            max_image_height = int(CARD_SIZE[1] - self.text_size[1] - 2*CARD_PADDING[1])
            self.image_pos[0] = self.get_center()[0] - self.image_size[0]/2
            self.image_pos[1] = self.pos[1] + max_image_height/2 + CARD_PADDING[1] - self.image_size[1]/2
#            self.image_pos[1] = self.get_center()[1] - self.text_size[1] - self.image_size[1]/2 + 2*CARD_PADDING[1]
        except:
            pass     
        
    #Set card's index in the board matrix    
    def set_index(self, index):            
        self.index = index
        
    def toggle_select(self):
        if self.selected:
            self.selected = False
        else:
            self.selected = True
 
    #Return the card's center position
    def get_center(self):
        return (self.pos[0] + CARD_SIZE[0]/2, self.pos[1] + CARD_SIZE[1]/2) 
        
    #Render some text in a wrapped text rectangle that fits within the card    
    def render_text_rec(self, text):
        text_size = self.font.size(text)
        if text_size[0] <= (CARD_SIZE[0] - 2*CARD_PADDING[0]):
            text_lines = 1;
        else:
            text_lines = 2;
        text_rect = pygame.Rect(0, 0, CARD_SIZE[0] - 2*CARD_PADDING[0], 1+text_lines*text_size[1])
        return render_textrect(text, self.font, text_rect, self.text_color, self.text_bgcolor, 1)       
    
    #Test to see if a given position (x,y) is covered by this card, used to check if clicked on card, etc.
    def test_pos_on_card(self, pos):
        if pos[0] > self.pos[0] and pos[0] < self.pos[0] + CARD_SIZE[0]:
            if pos[1] > self.pos[1] and pos[1] < self.pos[1] + CARD_SIZE[1]:
                return True
        return False         
    
    #Test whether a particular card is next to this one on the board
    def test_next_to(self, card):
        if abs(self.index[0] - card.index[0]) == 1 and self.index[1] == card.index[1]:
            return True
        elif abs(self.index[1] - card.index[1]) == 1 and self.index[0] == card.index[0]:
            return True
        else:
            return False     
        
    def get_adjacent_indexes(self, board):
        row = self.index[0]
        col = self.index[1]
        adjacent_indexes = []
        if row != 0 and (row - 1, col):
            adjacent_indexes.append((row - 1, col))
        if row != board.matrix.last_row: 
            adjacent_indexes.append((row + 1, col))
        if col != 0:
            adjacent_indexes.append((row, col - 1))
        if col != board.matrix.last_col: 
            adjacent_indexes.append((row, col + 1))
        return adjacent_indexes
                
    #Test whether there is a connection between a particular card and this one    
    #Returns True or False
    def test_connection(self, card, board):
        if self.type == BLANK or card.type == BLANK:
            return False
        if self.type == COUNTRY:
            if card.type == COUNTRY:   
                for country_name in self.borders:
                    if country_name == card.country_name:
                        return True
            elif card.type == TRANSPORT:
                if card.transport_type == SHIP:
                    for ocean_name in self.oceans:
                        if ocean_name == card.ocean_name:
                            return True
                elif card.transport_type == PLANE:
                    if self.color == card.color:
                        return True
                elif card.transport_type == CAR:
                    second_card_indexes = []
                    if card.index[1] > self.index[1]:
                        second_card_indexes.append((card.index[0], card.index[1] + 1))
                        second_card_indexes.append((card.index[0] + 1, card.index[1]))
                        second_card_indexes.append((card.index[0] - 1, card.index[1]))
                    elif card.index[1] < self.index[1]:
                        second_card_indexes.append((card.index[0], card.index[1] - 1))
                        second_card_indexes.append((card.index[0] + 1, card.index[1]))
                        second_card_indexes.append((card.index[0] - 1, card.index[1]))
                    elif card.index[0] > self.index[0]:
                        second_card_indexes.append((card.index[0] + 1, card.index[1]))
                        second_card_indexes.append((card.index[0], card.index[1] + 1))
                        second_card_indexes.append((card.index[0], card.index[1] - 1))
                    else:
                        second_card_indexes.append((card.index[0] + -1, card.index[1]))
                        second_card_indexes.append((card.index[0], card.index[1] + 1))
                        second_card_indexes.append((card.index[0], card.index[1] - 1))
                    for second_card_index in second_card_indexes:
                        if board.test_index_on_board(second_card_index):
                            second_card = board.get_card(second_card_index)
                            if second_card.type == COUNTRY:
                                for country_name in self.borders:
                                    for second_country_name in second_card.borders:
                                        if country_name == second_country_name or country_name == second_card.country_name:
                                            return True
        elif self.type == TRANSPORT:
            if card.type == TRANSPORT:
                return False
            elif card.type == COUNTRY:
                if self.transport_type == SHIP:
                    for ocean_name in card.oceans:
                        if ocean_name == self.ocean_name:
                            return True        
                elif self.transport_type == PLANE:
                    if self.color == card.color:
                        return True       
                elif self.transport_type == CAR:
                    second_card_indexes = []
                    if self.index[1] > card.index[1]:
                        second_card_indexes.append((self.index[0], self.index[1] + 1))
                        second_card_indexes.append((self.index[0] + 1, self.index[1]))
                        second_card_indexes.append((self.index[0] - 1, self.index[1]))
                    elif self.index[1] < card.index[1]:
                        second_card_indexes.append((self.index[0], self.index[1] - 1))
                        second_card_indexes.append((self.index[0] + 1, self.index[1]))
                        second_card_indexes.append((self.index[0] - 1, self.index[1]))
                    elif self.index[0] > card.index[0]:
                        second_card_indexes.append((self.index[0] + 1, self.index[1]))
                        second_card_indexes.append((self.index[0], self.index[1] + 1))
                        second_card_indexes.append((self.index[0], self.index[1] - 1))
                    else:
                        second_card_indexes.append((self.index[0] + -1, self.index[1]))
                        second_card_indexes.append((self.index[0], self.index[1] + 1))
                        second_card_indexes.append((self.index[0], self.index[1] - 1))
                    for second_card_index in second_card_indexes:
                        if board.test_index_on_board(second_card_index):
                            second_card = board.get_card(second_card_index)
                            if second_card.type == COUNTRY:
                                for country_name in card.borders:
                                    for second_country_name in second_card.borders:
                                        if country_name == second_country_name or country_name == second_card.country_name:
                                            return True
                      
        return False         
    
    #Test whether this card has any connections, in it's current location
    #Returns True or False
    def test_any_connections(self, board):
        r = self.index[0]
        c = self.index[1]
        try: 
            card_right = board.get_card([r, c+1])
            if self.test_connection(card_right, board):
                return True
        except:
            pass
        try: 
            card_down = board.get_card([r+1, c])
            if self.test_connection(card_down, board):
                return True
        except:
            pass
        try: 
            card_left = board.get_card([r, c-1])
            if self.test_connection(card_left, board):
                return True
        except:
            pass  
        try: 
            card_up = board.get_card([r-1, c])
            if self.test_connection(card_up, board):
                return True
        except:
            pass    
        return False   
    
    #Replace this card with another card
    def replace(self, card):
        card.set_pos(self.pos)
        card.set_index(self.index)
    
    def __repr__(self):
        return str(self.text)    
    
    def render(self, surface):  
        if self.selected:
            if self.image:
                pygame.draw.rect(surface, (0,0,0), Rect(self.pos, (CARD_SIZE[0], CARD_SIZE[1])))
                surface.blit(self.rendered_image, self.image_pos, None, 0)
        else:
            if self.rendered_image:
                #Background color
                pygame.draw.rect(surface, WHITE, Rect(self.pos, (CARD_SIZE[0], CARD_SIZE[1])))
                #Image
                surface.blit(self.rendered_image, self.image_pos, None, 0)
                #Outline
                pygame.draw.rect(surface, BLACK, Rect(self.pos, (CARD_SIZE[0], CARD_SIZE[1])), 1)
            else:
                pygame.draw.rect(surface, self.color, Rect(self.pos, (CARD_SIZE[0], CARD_SIZE[1])))
        #Text
        surface.blit(self.rendered_text, self.text_pos)
           
        #Price 
        
        try:
            surface.blit(self.rendered_price, self.price_pos)
        except Exception, e:
            print e

    
class BlankCard(Card):
        
    def __init__(self):
        self.price = 0
        self.borders = None
        # Call the base class constructor
        Card.__init__(self, BLANK, "Blank", "assets/images/blank_card.png", RED)     
    
class CountryCard(Card):
        
    def __init__(self, country):
        self.country = country
        self.country_name = country.name
        self.borders = country.borders
        self.oceans = country.oceans
        self.country_size = country.size
        self.price = country.price        
        # Call the base class constructor
        Card.__init__(self, COUNTRY, country.name, country.image, country.color)  
        
class TransportCard(Card):
        
    def __init__(self, transport_type, text, image, color):
        # Call the base class constructor
        Card.__init__(self, TRANSPORT, text, image, color)  
        self.transport_type = transport_type
        
class PlaneCard(TransportCard):
        
    def __init__(self, color):
        plane_image = 'assets/images/transport/plane.png'
        self.price = 100
        # Call the base class constructor
        TransportCard.__init__(self, PLANE, "Airplane", plane_image, color)    
        
class TrainCard(TransportCard):
        
    def __init__(self):
        train_image = 'assets/images/transport/train.png'
        # Call the base class constructor
        TransportCard.__init__(self, TRAIN, "Railroad", train_image, None)   
        
class CarCard(TransportCard):
        
    def __init__(self):
        car_image = 'assets/images/transport/car.png'
        self.price = 200          
        # Call the base class constructor
        TransportCard.__init__(self, CAR, "Car", car_image, None)     
        
class ShipCard(TransportCard):
        
    def __init__(self, ocean):
        ship_image = 'assets/images/transport/ship.png'
        self.ocean = ocean
        self.ocean_name = ocean.name       
        self.price = len(ocean.borders)*10              
        # Call the base class constructor
        TransportCard.__init__(self, SHIP, ocean.name + " Ship", ship_image, None)                      
    
class Connection(object):    
    
    def __init__(self, card1, card2):
        self.card1 = card1
        self.card2 = card2
        self.point_a = card1.get_center()
        self.point_b = card2.get_center()
        
        
        
    def __repr__(self):
        return self.card1.name + "--" + self.card2.name
    
    def render(self, surface):
        pygame.draw.aaline(surface, (0, 0, 0), self.point_a, self.point_b)
    
#Run game 
def main():
    global global_continent
    pygame.init()
    screen = pygame.display.set_mode(SCREEN_SIZE)
    continent = World().europe
    interface = Interface(continent)
        
    #Main game loop    
    while True:  
                            
        interface.process()     
        
        interface.render(screen)
        
        pygame.display.update()

# this runs the main function if this script is called to run.
#  If it is imported as a module, we don't run the main function.
if __name__ == "__main__":
    main()