import pygame
from pygame.locals import *
from events import *

#vectorSum = lambda a,b: (a[0]+b[0], a[1]+b[1])

#------------------------------------------------------------------------------
class Widget(pygame.sprite.Sprite):
    def __init__(self, ev_manager, pos, container=None):
        pygame.sprite.Sprite.__init__(self)

        self.ev_manager = ev_manager
        self.ev_manager.register_listener( self )
        
        self.pos = pos

        self.container = container
        self.focused = 0
        self.dirty = 1

    #----------------------------------------------------------------------
    def set_focus(self, val):
        self.focused = val
        self.dirty = 1

    #----------------------------------------------------------------------
    def kill(self):
        self.container = None
        del self.container
        pygame.sprite.Sprite.kill(self)

    #----------------------------------------------------------------------
    def notify(self, event):

        if isinstance(event, GUIFocusThisWidgetEvent) and event.widget is self:
            self.set_focus(1)

#        elif isinstance(event, GUIFocusThisWidgetEvent) and self.focused:
#            self.set_focus(0)
            
        elif isinstance(event, GUIUnfocusThisWidgetEvent) and event.widget is self:
            self.set_focus(0)            
            
#------------------------------------------------------------------------------
class ButtonSprite(Widget):
    def __init__(self, ev_manager, text, pos, container=None, on_click_event=None ):
        Widget.__init__( self, ev_manager, pos, container)

        self.font = pygame.font.Font(None, 30)
        self.text = text
        self.image = self.font.render( self.text, 1, (255,0,0))
        self.rect  = self.image.get_rect()
        self.rect.topleft = self.pos

        self.on_click_event = on_click_event

    #----------------------------------------------------------------------
    def update(self):
        if not self.dirty:
            return

        if self.focused:
            color = (255,255,0)
        else:
            color = (255,0,0)
        self.image = self.font.render(self.text, 1, color)
        self.rect  = self.image.get_rect()
        self.rect.topleft = self.pos

        self.dirty = 0

    #----------------------------------------------------------------------
    def connect(self, event_dict):
        for key, event in event_dict.iteritems():
            try:
                self.__setattr__( key, event )
            except AttributeError:
                print "Couldn't connect the ", key
                pass

    #----------------------------------------------------------------------
    def click(self):
        self.dirty = 1
        if self.on_click_event:
            self.ev_manager.post(self.on_click_event)

    #----------------------------------------------------------------------
    def render(self, surface):
        self.update()
        surface.blit(self.image, self.pos)


    #----------------------------------------------------------------------
    def notify(self, event):
        if isinstance(event, MouseButtonLeftEvent) and self.focused:
            self.click()

        elif isinstance(event, MouseMoveEvent) and self.rect.collidepoint(event.pos):
            ev = GUIFocusThisWidgetEvent(self)
            self.ev_manager.post(ev)       
            
        elif isinstance(event, MouseMoveEvent) and not self.rect.collidepoint(event.pos):
            ev = GUIUnfocusThisWidgetEvent(self)
            self.ev_manager.post(ev)                     

        Widget.notify(self, event)
            