import string

COLOR1 = (253, 221, 1)      #Yellow
COLOR2 = (255, 0, 240)      #Pink  
COLOR3 = (253, 90, 1)       #Orange
COLOR4 = (1, 131, 253)      #Blue
COLOR5 = (7, 156, 0)        #Green

BIG = 0.9
MED = 0.75
SMALL = 0.5
TINY = 0.3

class NoSuchCountry(Exception):
    
    def __init__(self, value):
        self.value = value
        
    def __str__(self):
        return repr(self.value)
    
class NoSuchOcean(Exception):
    
    def __init__(self, value):
        self.value = value
        
    def __str__(self):
        return repr(self.value)    
    
class IntegrityProblem(Exception):
    
    def __init__(self, value):
        self.value = value
        
    def __str__(self):
        return repr(self.value)    
    
class Country(object):
    
    def __init__(self, name, size, color, borders, oceans):
        self.name = name
        self.size = size
        self.color = color
        self.borders = borders
        self.oceans = oceans
        self.price = 10 * (len(self.borders) + len(self.oceans))
        self.number = 1;    #Number of this country to put in a deck
        
        self.machine_name = string.lower(string.replace(name, ' ', '-')) 
        self.image = "assets/images/countries/europe/" + self.machine_name + ".png"
        
    def __repr__(self):
        return str(self.name)    

class Ocean(object):
    
    def __init__(self, name, number, borders):
        self.name = name
        self.number = number    #Number of this ocean to put in a deck
        self.borders = borders
        
    def __repr__(self):
        return str(self.name)        

class Continent(object):
    
    def __init__(self):
        self.name = ''
        self.countries = []
        self.oceans = []
        pass

    def __repr__(self):
        return str(self.countries)      
        
    #Return the country with the given name
    def get_country(self, name):
        for country in self.countries:
            if country.name == name:
                return country
        #return False
        raise NoSuchCountry('No such country as ' + name + ' in ' + self.name)
    
    #Return the ocean with the given name
    def get_ocean(self, name):
        for ocean in self.oceans:
            if ocean.name == name:
                return ocean
        #return False
        raise NoSuchOcean('No such ocean as ' + name + ' in ' + self.name)    
    
    #Tests the country connections and ocean within this continent to make sure they match up
    def test_integrity(self):
        #Test country borders
        print '******************************'
        print 'Checking country borders...'
        print '******************************'
        for country in self.countries:
            for border_name in country.borders:
                bordering_country = self.get_country(border_name)
                for bordering_country_border_name in bordering_country.borders:
                    if bordering_country_border_name == country.name:  
                        match_found = True
                if match_found:
                    print 'Checked: ' + country.name + ' - ' + bordering_country.name
                    match_found = False
                    continue
                else:
                    raise IntegrityProblem(country.name + ' borders ' + bordering_country.name + ', but there is such border indicated in ' + bordering_country.name)
                
        #Test ocean borders, ocean-side
        print '******************************'
        print 'Checking oceans, ocean-side...'
        print '******************************'
        for ocean in self.oceans:
            for border_name in ocean.borders:
                bordering_country = self.get_country(border_name)
                for bordering_country_ocean_name in bordering_country.oceans:
                    if bordering_country_ocean_name == ocean.name:  
                        match_found = True 
                if match_found:
                    print 'Checked: ' + ocean.name + ' - ' + bordering_country.name
                    match_found = False
                    continue
                else:
                    raise IntegrityProblem(ocean.name + ' borders ' + bordering_country.name + ', but there is such ocean indicated in ' + bordering_country.name)

        #Test ocean borders, country-side
        print '******************************'
        print 'Checking oceans, country-side...'  
        print '******************************'      
        for country in self.countries:
            for ocean_name in country.oceans:
                bordering_ocean = self.get_ocean(ocean_name)
                for bordering_ocean_country_name in bordering_ocean.borders:
                    if bordering_ocean_country_name == country.name:  
                        match_found = True
                if match_found:
                    print 'Checked: ' + country.name + ' - ' + bordering_ocean.name
                    match_found = False
                    continue
                else:
                    raise IntegrityProblem(country.name + ' borders ' + bordering_ocean.name + ', but there is such border indicated in ' + bordering_ocean.name)                
                                                                    
class Europe(Continent):
    
    def __init__(self):
        # Call the base class constructor
        Continent.__init__(self)     
           
        self.name = 'Europe'
        
        color = COLOR1;
        self.countries.append(Country("Austria", MED, color, ["Switzerland", "Germany", "Czech Republic", "Slovakia", "Hungary", "Slovenia", "Italy"], []))
        self.countries.append(Country("Spain", BIG, color, ["Portugal", "France"], ["Med", "Atlantic"]))
        self.countries.append(Country("Belgium", SMALL, color, ["France", "The Netherlands", "Luxemburg", "Germany"], ["Atlantic"]))
        self.countries.append(Country("Greece", BIG, color, ["Albania", "Macedonia", "Bulgaria", "Turkey"], ["Med"]))
        self.countries.append(Country("Romania", BIG, color, ["Bulgaria", "Serbia", "Hungary", "Ukraine", "Moldova", "Turkey"], []))        
        self.countries.append(Country("Belarus", BIG, color, ["Ukraine", "Poland", "Lithuania", "Latvia", "Russia"], []))
        self.countries.append(Country("Northern Ireland", SMALL, color, ["Ireland", "Scotland"], ["Atlantic"]))
        self.countries.append(Country("Finland", BIG, color, ["Russia", "Sweden", "Norway"], ["Baltic"]))
        self.countries.append(Country("Montenegro", SMALL, color, ["Bosnia & Herzegovina", "Serbia", "Albania"], ["Med"])) 
        
        color = COLOR2;
        self.countries.append(Country("France", BIG, color, ["England", "Spain", "Belgium", "Luxemburg", "Germany", "Switzerland", "Italy"], ["Med", "Atlantic"]))
        self.countries.append(Country("Sweden", BIG, color, ["Finland", "Norway", "Denmark"], ["Baltic", "Atlantic"]))
        self.countries.append(Country("Czech Republic", MED, color, ["Germany", "Poland", "Slovakia", "Austria"], []))
        self.countries.append(Country("Slovenia", SMALL, color, ["Italy", "Austria", "Hungary", "Croatia"], ["Med"]))
        self.countries.append(Country("Serbia", MED, color, ["Bosnia & Herzegovina", "Croatia", "Hungary", "Romania", "Bulgaria", "Macedonia", "Albania", "Montenegro"], []))
        self.countries.append(Country("Ukraine", BIG, color, ["Bulgaria", "Turkey", "Romania", "Moldova", "Hungary", "Slovakia", "Poland", "Belarus", "Russia"], []))
        self.countries.append(Country("Lithuania", MED, color, ["Poland", "Belarus", "Latvia"], ["Baltic"]))
        self.countries.append(Country("Scotland", MED, color, ["England", "Northern Ireland"], ["Atlantic"]))
        
        color = COLOR3;
        self.countries.append(Country("Portugal", MED, color, ["Spain"], ["Atlantic"]))
        self.countries.append(Country("Germany", BIG, color, ["Switzerland", "Luxemburg", "Belgium", "France", "The Netherlands", "Denmark", "Poland", "Czech Republic", "Austria"], ["Atlantic", "Baltic"]))
        self.countries.append(Country("England", MED, color, ["Wales", "Scotland", "France", "Ireland"], ["Atlantic"]))
        self.countries.append(Country("Hungary", MED, color, ["Croatia", "Slovenia", "Austria", "Slovakia", "Ukraine", "Romania", "Serbia"], []))
        self.countries.append(Country("Bosnia & Herzegovina", MED, color, ["Croatia", "Serbia", "Montenegro"], ["Med"]))
        self.countries.append(Country("Bulgaria", MED, color, ["Turkey", "Greece", "Macedonia", "Serbia", "Romania", "Ukraine"], []))
        self.countries.append(Country("Russia", BIG, color, ["Finland", "Estonia", "Latvia", "Belarus", "Ukraine", "Norway"], ["Baltic", "Atlantic"]))
        
        color = COLOR4;
        self.countries.append(Country("Turkey", BIG, color, ["Greece", "Bulgaria", "Romania", "Ukraine"], ["Med"]))  
        self.countries.append(Country("Macedonia", SMALL, color, ["Greece", "Albania", "Serbia", "Bulgaria"], []))
        self.countries.append(Country("Italy", BIG, color, ["France", "Switzerland", "Austria", "Slovenia"], ["Med"]))
        self.countries.append(Country("Luxemburg", TINY, color, ["France", "Belgium", "Germany"], []))
        self.countries.append(Country("Ireland", MED, color, ["Northern Ireland", "England"], ["Atlantic"]))
        self.countries.append(Country("Denmark", MED, color, ["Germany", "Sweden"], ["Baltic", "Atlantic"]))
        self.countries.append(Country("Estonia", SMALL, color, ["Russia", "Latvia"], ["Baltic"]))   
        self.countries.append(Country("Poland", BIG, color, ["Belarus", "Ukraine", "Slovakia", "Czech Republic", "Germany", "Lithuania"], ["Baltic"]))
        self.countries.append(Country("Moldova", SMALL, color, ["Ukraine", "Romania"], [])) 
        
        color = COLOR5;
        self.countries.append(Country("Switzerland", SMALL, color, ["Italy", "France", "Germany", "Austria"], []))  
        self.countries.append(Country("The Netherlands", MED, color, ["Belgium", "Germany"], ["Atlantic"]))
        self.countries.append(Country("Wales", SMALL, color, ["England"], ["Atlantic"]))
        self.countries.append(Country("Norway", BIG, color, ["Sweden", "Finland", "Russia"], ["Atlantic"]))
        self.countries.append(Country("Latvia", MED, color, ["Estonia", "Russia", "Lithuania", "Belarus"], ["Baltic"]))
        self.countries.append(Country("Slovakia", MED, color, ["Hungary", "Austria", "Czech Republic", "Poland", "Ukraine"], []))
        self.countries.append(Country("Albania", SMALL, color, ["Greece", "Macedonia", "Serbia", "Montenegro"], ["Med"]))   
        self.countries.append(Country("Croatia", MED, color, ["Bosnia & Herzegovina", "Serbia", "Hungary", "Slovenia"], ["Med"]))         
        
        #self.countries.append(Country("Test", color, ["Austria", "Spain", "Greece", "Romania"], ["Baltic"]))        
        
        #Oceans
        self.oceans.append(Ocean("Atlantic", 3, ["Sweden", "Russia", "Norway", "Denmark", "Germany", "The Netherlands", "Belgium", "France", "England", "Scotland", "Northern Ireland", "Ireland", "Wales", "Spain", "Portugal"]))
        self.oceans.append(Ocean("Baltic", 1, ["Sweden", "Finland", "Russia", "Estonia", "Latvia", "Lithuania", "Poland", "Germany", "Denmark"]))
        self.oceans.append(Ocean("Med", 2, ["Turkey", "Greece", "Albania", "Montenegro", "Bosnia & Herzegovina", "Croatia", "Slovenia", "Italy", "France", "Spain"]))
        
        #Colors
        self.colors = [COLOR1, COLOR2, COLOR3, COLOR4, COLOR5]
        
        self.size = len(self.countries)
        self.map = "map_europe.png"
        
class World(object):
    
    def __init__(self):
        self.europe = Europe()
        self.countries = self.europe.countries
        self.size = self.europe.size
        
    def __repr__(self):
        return str(self.europe)
            
# this runs the main function if this script is called to run.
#  If it is imported as a module, we don't run the main function.
if __name__ == "__main__":
    europe = World().europe
    europe.test_integrity()
        